@@ -486,11 +486,32 @@ void
 dGenXtrue_dist(int_t n, int_t nrhs, double *x, int_t ldx)
 {
     int  i, j;
-    for (j = 0; j < nrhs; ++j)
+    double exponent, tau; /* See TOMS paper on ItRef (LAWN165); testing code: 
+			     Codes/UCB-itref-xblas-etc/xiaoye/itref/driver.c  */
+    double r;
+    
+    exponent = (double)rand() / (double)((unsigned)RAND_MAX + 1); /* uniform in [0,1) */
+#if 1
+    tau = pow(2.0, 12.0 * exponent);
+#else
+    tau = 5.0;
+#endif
+    //printf("new dGenXtrue, tau %e\n", tau);
+    
+    r = (double)rand() / (double)((unsigned)RAND_MAX + 1); /* uniform in [0,1) */
+    r = r + 0.5; /* uniform in (0.5, 1.5) */
+
+    for (j = 0; j < nrhs; ++j) {
 	for (i = 0; i < n; ++i) {
-	    if ( i % 2 ) x[i + j*ldx] = 1.0 + (double)(i+1.)/n;
-	    else x[i + j*ldx] = 1.0 - (double)(i+1.)/n;
+#if 1
+	  x[i + j*ldx] = (double) pow(tau, - ((double)i / (n-1))) * r;
+
+	  //if (i % 2) x[i + j*ldx] = 1.0; else x[i + j*ldx] = -1.0;
+#else
+	  x[i + j*ldx] = (double)rand() / (double)((unsigned)RAND_MAX + 1); /* uniform in [0,1) */
+#endif
 	}
+    }
 }
 
 /*! \brief Let rhs[i] = sum of i-th row of A, so the solution vector is all 1's
@@ -546,10 +567,10 @@ void Printdouble5(char *name, int_t len, double *x)
 
     printf("%10s:", name);
     for (i = 0; i < len; ++i) {
-	if ( i % 5 == 0 ) printf("\n[%ld-%ld] ", (long int) i, (long int) i+4);
-	printf("%14e", x[i]);
+	if ( i % 5 == 0 ) printf("\n[%d-%d] ", (int) i, (int) i+4);
+	printf("%20.16e ", x[i]);
     }
-    printf("\n");
+    printf("\n\n");
 }
 
 int file_Printdouble5(FILE *fp, char *name, int_t len, double *x)
@@ -605,11 +626,11 @@ void dPrintLblocks(int iam, int_t nsupers, gridinfo_t *grid,
 	}
 	printf("(%d)", iam);
  	PrintInt32("ToSendR[]", grid->npcol, Llu->ToSendR[lb]);
-	PrintInt10("fsendx_plist[]", grid->nprow, Llu->fsendx_plist[lb]);
+	PrintInt32("fsendx_plist[]", grid->nprow, Llu->fsendx_plist[lb]);
     }
-    printf("nfrecvx " IFMT "\n", Llu->nfrecvx);
+    printf("nfrecvx %d\n", Llu->nfrecvx);
     k = CEILING( nsupers, grid->nprow );
-    PrintInt10("fmod", k, Llu->fmod);
+    PrintInt32("fmod", k, Llu->fmod);
 
 } /* DPRINTLBLOCKS */
 
@@ -745,6 +766,464 @@ void dDumpLblocks(int iam, int_t nsupers, gridinfo_t *grid,
 } /* dDumpLblocks */
 
 
+
+
+/*! \Compute the level sets in the L factor
+ */
+void dComputeLevelsets(int iam, int_t nsupers, gridinfo_t *grid,
+		  Glu_persist_t *Glu_persist, dLocalLU_t *Llu, int_t *levels)
+{
+    register int c, extra, gb, j, i, lb, nsupc, nsupr, len, nb, ncb;
+    register int_t k, mycol, r;
+	int_t nnzL, n,nmax,lk;
+    int_t *xsup = Glu_persist->xsup;
+    int_t *index,*lloc;
+    double *nzval;
+	char filename[256];
+	FILE *fp, *fopen();
+
+	// assert(grid->npcol*grid->nprow==1);
+
+	// count nonzeros in the first pass
+	nnzL = 0;
+	n = 0;
+    ncb = nsupers / grid->npcol;
+    extra = nsupers % grid->npcol;
+    mycol = MYCOL( iam, grid );
+    if ( mycol < extra ) ++ncb;
+    for (lb = 0; lb < ncb; ++lb) {
+	index = Llu->Lrowind_bc_ptr[lb];
+	if ( index ) { /* Not an empty column */
+	    nzval = Llu->Lnzval_bc_ptr[lb];
+        lloc = Llu->Lindval_loc_bc_ptr[lb];
+	    nb = index[0];
+
+
+	    for (c = 0; c < nb; ++c) {
+        lk=lloc[c];
+		levels[lk]=SUPERLU_MAX(levels[lb]+1,levels[lk]);
+	    }
+	}
+    }
+
+
+} /* dComputeLevelsets */
+
+
+/*! \Dump the factored matrix L using matlab triple-let format
+ */
+void dGenCOOLblocks(int iam, int_t nsupers, gridinfo_t *grid,
+		  Glu_persist_t *Glu_persist, dLocalLU_t *Llu, int_t** cooRows, int_t** cooCols, double ** cooVals, int_t* n, int_t* nnzL)
+{
+    register int c, extra, gb, j, i, lb, nsupc, nsupr, len, nb, ncb;
+    register int_t k, mycol, r;
+	int_t nmax,cnt;
+    int_t *xsup = Glu_persist->xsup;
+    int_t *index;
+    double *nzval;
+	FILE *fp, *fopen();
+
+	assert(grid->npcol*grid->nprow==1);
+
+	 
+	// count nonzeros in the first pass
+	*nnzL = 0;
+	*n = 0;
+	ncb = nsupers / grid->npcol;
+	extra = nsupers % grid->npcol;
+	mycol = MYCOL( iam, grid );
+	if ( mycol < extra ) ++ncb;
+	for (lb = 0; lb < ncb; ++lb) {
+	index = Llu->Lrowind_bc_ptr[lb];
+	if ( index ) { /* Not an empty column */
+
+		nzval = Llu->Lnzval_bc_ptr[lb];
+		nb = index[0];
+		nsupr = index[1];
+		gb = lb * grid->npcol + mycol;
+		nsupc = SuperSize( gb );
+		for (c = 0, k = BC_HEADER, r = 0; c < nb; ++c) {
+		len = index[k+1];
+
+		for (j = 0; j < nsupc; ++j) {
+		for (i=0; i<len; ++i){
+
+		if(index[k+LB_DESCRIPTOR+i]+1>=xsup[gb]+j+1){			
+			(*nnzL) ++;
+			nmax = SUPERLU_MAX(*n,index[k+LB_DESCRIPTOR+i]+1);
+			*n = nmax;
+		}
+
+		}
+		}
+		k += LB_DESCRIPTOR + len;
+		r += len;
+		}
+	}
+	} 
+				
+	// fill the triplets in the second pass
+    if ( !(*cooRows = (int_t*)SUPERLU_MALLOC(*nnzL * sizeof(int_t))) )
+        ABORT("Malloc fails for cooRows[].");
+    if ( !(*cooCols = (int_t*)SUPERLU_MALLOC(*nnzL * sizeof(int_t))) )
+        ABORT("Malloc fails for cooCols[].");
+    if ( !(*cooVals = (double*)SUPERLU_MALLOC(*nnzL * sizeof(double))) )
+        ABORT("Malloc fails for cooVals[].");
+	*nnzL = 0;
+	*n = 0;
+	ncb = nsupers / grid->npcol;
+	extra = nsupers % grid->npcol;
+	mycol = MYCOL( iam, grid );
+	if ( mycol < extra ) ++ncb;
+	for (lb = 0; lb < ncb; ++lb) {
+	index = Llu->Lrowind_bc_ptr[lb];
+	if ( index ) { /* Not an empty column */
+
+		nzval = Llu->Lnzval_bc_ptr[lb];
+		nb = index[0];
+		nsupr = index[1];
+		gb = lb * grid->npcol + mycol;
+		nsupc = SuperSize( gb );
+		for (c = 0, k = BC_HEADER, r = 0; c < nb; ++c) {
+		len = index[k+1];
+
+		for (j = 0; j < nsupc; ++j) {
+		for (i=0; i<len; ++i){
+
+		if(index[k+LB_DESCRIPTOR+i]+1>=xsup[gb]+j+1){
+            (*cooRows)[(*nnzL)]=index[k+LB_DESCRIPTOR+i];
+            (*cooCols)[(*nnzL)]=xsup[gb]+j;
+            if((*cooRows)[(*nnzL)]==(*cooCols)[(*nnzL)]){
+                (*cooVals)[(*nnzL)]=1.0;
+            }else{
+                (*cooVals)[(*nnzL)]=nzval[r +i+ j*nsupr];								
+            }
+			
+			(*nnzL) ++;
+			nmax = SUPERLU_MAX(*n,index[k+LB_DESCRIPTOR+i]+1);
+			*n = nmax;
+		}
+
+		}
+		}
+		k += LB_DESCRIPTOR + len;
+		r += len;
+		}
+	}
+	} 
+
+} /* dGenCOOLblocks */
+
+
+
+
+/*! \Dump the factored matrix L using CSC format
+ */
+void dGenCSCLblocks(int iam, int_t nsupers, gridinfo_t *grid,
+		  Glu_persist_t *Glu_persist, dLocalLU_t *Llu, double **nzval, int_t **rowind, int_t **colptr, int_t* n, int_t* nnzL)
+{
+    register int c, extra, gb, j, i, lb, nsupc, nsupr, len, nb, ncb;
+    register int_t k, mycol, r;
+	int_t nmax,cnt, jsize;
+    int_t *xsup = Glu_persist->xsup;
+    int_t *index;
+    double *nzval0;
+	FILE *fp, *fopen();
+    
+    double *val;
+    int_t  *row, *col;
+
+    double *a;
+    int_t    *asub, *xa;
+    int_t nz;
+
+
+	assert(grid->npcol*grid->nprow==1);
+
+	// count nonzeros in the first pass
+	*nnzL = 0;
+	*n = 0;
+	ncb = nsupers / grid->npcol;
+	extra = nsupers % grid->npcol;
+	mycol = MYCOL( iam, grid );
+	if ( mycol < extra ) ++ncb;
+	for (lb = 0; lb < ncb; ++lb) {
+	index = Llu->Lrowind_bc_ptr[lb];
+	if ( index ) { /* Not an empty column */
+
+		nzval0 = Llu->Lnzval_bc_ptr[lb];
+		nb = index[0];
+		nsupr = index[1];
+		gb = lb * grid->npcol + mycol;
+		nsupc = SuperSize( gb );
+		for (c = 0, k = BC_HEADER, r = 0; c < nb; ++c) {
+		len = index[k+1];
+
+		for (j = 0; j < nsupc; ++j) {
+		for (i=0; i<len; ++i){
+
+		if(index[k+LB_DESCRIPTOR+i]+1>=xsup[gb]+j+1){			
+			(*nnzL) ++;
+			nmax = SUPERLU_MAX(*n,index[k+LB_DESCRIPTOR+i]+1);
+			*n = nmax;
+		}
+
+		}
+		}
+		k += LB_DESCRIPTOR + len;
+		r += len;
+		}
+	}
+	} 
+				
+	// get triplelets in the second pass
+    if ( !(val = (double *) SUPERLU_MALLOC(*nnzL * sizeof(double))) )
+        ABORT("Malloc fails for val[]");
+    if ( !(row = (int_t *) SUPERLU_MALLOC(*nnzL * sizeof(int_t))) )
+        ABORT("Malloc fails for row[]");
+    if ( !(col = (int_t *) SUPERLU_MALLOC(*nnzL * sizeof(int_t))) )
+        ABORT("Malloc fails for col[]");
+	*nnzL = 0;
+	*n = 0;
+	ncb = nsupers / grid->npcol;
+	extra = nsupers % grid->npcol;
+	mycol = MYCOL( iam, grid );
+	if ( mycol < extra ) ++ncb;
+	for (lb = 0; lb < ncb; ++lb) {
+	index = Llu->Lrowind_bc_ptr[lb];
+	if ( index ) { /* Not an empty column */
+
+		nzval0 = Llu->Lnzval_bc_ptr[lb];
+		nb = index[0];
+		nsupr = index[1];
+		gb = lb * grid->npcol + mycol;
+		nsupc = SuperSize( gb );
+		for (c = 0, k = BC_HEADER, r = 0; c < nb; ++c) {
+		len = index[k+1];
+
+		for (j = 0; j < nsupc; ++j) {
+		for (i=0; i<len; ++i){
+
+		if(index[k+LB_DESCRIPTOR+i]+1>=xsup[gb]+j+1){
+            row[(*nnzL)]=index[k+LB_DESCRIPTOR+i];
+            col[(*nnzL)]=xsup[gb]+j;
+            if(row[(*nnzL)]==col[(*nnzL)]){
+                val[(*nnzL)]=1.0;
+            }else{
+                val[(*nnzL)]=nzval0[r +i+ j*nsupr];								
+            }
+			
+			(*nnzL) ++;
+			nmax = SUPERLU_MAX(*n,index[k+LB_DESCRIPTOR+i]+1);
+			*n = nmax;
+		}
+		}
+		}
+		k += LB_DESCRIPTOR + len;
+		r += len;
+		}
+	}
+	}
+
+
+    dallocateA_dist(*n, *nnzL, nzval, rowind, colptr); /* Allocate storage */
+    a    = *nzval;
+    asub = *rowind;
+    xa   = *colptr;
+
+
+    for (j = 0; j < *n; ++j) xa[j] = 0;
+    /* Scan the triplet array to get nonzeros per column */
+    for (nz = 0; nz < *nnzL; ++nz) {
+	    ++xa[col[nz]];
+    }
+
+    /* Initialize the array of column pointers */
+    k = 0;
+    jsize = xa[0];
+    xa[0] = 0;
+    for (j = 1; j < *n; ++j) {
+	k += jsize;
+	jsize = xa[j];
+	xa[j] = k;
+    }
+
+    /* Copy the triplets into the column oriented storage */
+    for (nz = 0; nz < *nnzL; ++nz) {
+	j = col[nz];
+	k = xa[j];
+	asub[k] = row[nz];
+	a[k] = val[nz];
+	++xa[j];
+    }
+
+    /* Reset the column pointers to the beginning of each column */
+    for (j = *n; j > 0; --j)
+	xa[j] = xa[j-1];
+    xa[0] = 0;
+
+    SUPERLU_FREE(val);
+    SUPERLU_FREE(row);
+    SUPERLU_FREE(col);
+
+} /* dGenCSCLblocks */
+
+
+
+/*! \Dump the factored matrix L using CSR format
+ */
+void dGenCSRLblocks(int iam, int_t nsupers, gridinfo_t *grid,
+		  Glu_persist_t *Glu_persist, dLocalLU_t *Llu, double **nzval, int_t **colind, int_t **rowptr, int_t* n, int_t* nnzL)
+{
+    register int c, extra, gb, j, i, lb, nsupc, nsupr, len, nb, ncb;
+    register int_t k, mycol, r;
+	int_t nmax,cnt, isize;
+    int_t *xsup = Glu_persist->xsup;
+    int_t *index;
+    double *nzval0;
+	FILE *fp, *fopen();
+    
+    double *val;
+    int_t  *row, *col;
+
+    double *a;
+    int_t    *asub, *xa;
+    int_t nz;
+
+
+	assert(grid->npcol*grid->nprow==1);
+
+	// count nonzeros in the first pass
+	*nnzL = 0;
+	*n = 0;
+	ncb = nsupers / grid->npcol;
+	extra = nsupers % grid->npcol;
+	mycol = MYCOL( iam, grid );
+	if ( mycol < extra ) ++ncb;
+	for (lb = 0; lb < ncb; ++lb) {
+	index = Llu->Lrowind_bc_ptr[lb];
+	if ( index ) { /* Not an empty column */
+
+		nzval0 = Llu->Lnzval_bc_ptr[lb];
+		nb = index[0];
+		nsupr = index[1];
+		gb = lb * grid->npcol + mycol;
+		nsupc = SuperSize( gb );
+		for (c = 0, k = BC_HEADER, r = 0; c < nb; ++c) {
+		len = index[k+1];
+
+		for (j = 0; j < nsupc; ++j) {
+		for (i=0; i<len; ++i){
+
+		if(index[k+LB_DESCRIPTOR+i]+1>=xsup[gb]+j+1){			
+			(*nnzL) ++;
+			nmax = SUPERLU_MAX(*n,index[k+LB_DESCRIPTOR+i]+1);
+			*n = nmax;
+		}
+
+		}
+		}
+		k += LB_DESCRIPTOR + len;
+		r += len;
+		}
+	}
+	} 
+				
+	// get triplelets in the second pass
+    if ( !(val = (double *) SUPERLU_MALLOC(*nnzL * sizeof(double))) )
+        ABORT("Malloc fails for val[]");
+    if ( !(row = (int_t *) SUPERLU_MALLOC(*nnzL * sizeof(int_t))) )
+        ABORT("Malloc fails for row[]");
+    if ( !(col = (int_t *) SUPERLU_MALLOC(*nnzL * sizeof(int_t))) )
+        ABORT("Malloc fails for col[]");
+	*nnzL = 0;
+	*n = 0;
+	ncb = nsupers / grid->npcol;
+	extra = nsupers % grid->npcol;
+	mycol = MYCOL( iam, grid );
+	if ( mycol < extra ) ++ncb;
+	for (lb = 0; lb < ncb; ++lb) {
+	index = Llu->Lrowind_bc_ptr[lb];
+	if ( index ) { /* Not an empty column */
+
+		nzval0 = Llu->Lnzval_bc_ptr[lb];
+		nb = index[0];
+		nsupr = index[1];
+		gb = lb * grid->npcol + mycol;
+		nsupc = SuperSize( gb );
+		for (c = 0, k = BC_HEADER, r = 0; c < nb; ++c) {
+		len = index[k+1];
+
+		for (j = 0; j < nsupc; ++j) {
+		for (i=0; i<len; ++i){
+
+		if(index[k+LB_DESCRIPTOR+i]+1>=xsup[gb]+j+1){
+            row[(*nnzL)]=index[k+LB_DESCRIPTOR+i];
+            col[(*nnzL)]=xsup[gb]+j;
+            if(row[(*nnzL)]==col[(*nnzL)]){
+                val[(*nnzL)]=1.0;
+            }else{
+                val[(*nnzL)]=nzval0[r +i+ j*nsupr];								
+            }
+			
+			(*nnzL) ++;
+			nmax = SUPERLU_MAX(*n,index[k+LB_DESCRIPTOR+i]+1);
+			*n = nmax;
+		}
+		}
+		}
+		k += LB_DESCRIPTOR + len;
+		r += len;
+		}
+	}
+	}
+
+
+    dallocateA_dist(*n, *nnzL, nzval, colind, rowptr); /* Allocate storage */
+    a    = *nzval;
+    asub = *colind;
+    xa   = *rowptr;
+
+
+    for (i = 0; i < *n; ++i) xa[i] = 0;
+    /* Scan the triplet array to get nonzeros per row */
+    for (nz = 0; nz < *nnzL; ++nz) {
+	    ++xa[row[nz]];
+    }
+
+    /* Initialize the array of row pointers */
+    k = 0;
+    isize = xa[0];
+    xa[0] = 0;
+    for (i = 1; i < *n; ++i) {
+	k += isize;
+	isize = xa[i];
+	xa[i] = k;
+    }
+
+    /* Copy the triplets into the row oriented storage */
+    for (nz = 0; nz < *nnzL; ++nz) {
+	i = row[nz];
+	k = xa[i];
+	asub[k] = col[nz];
+	a[k] = val[nz];
+	++xa[i];
+    }
+
+    /* Reset the row pointers to the beginning of each row */
+    for (i = *n; i > 0; --i)
+	xa[i] = xa[i-1];
+    xa[0] = 0;
+
+    SUPERLU_FREE(val);
+    SUPERLU_FREE(row);
+    SUPERLU_FREE(col);
+
+} /* dGenCSRLblocks */
+
+
+
+
 /*! \brief Print the blocks in the factored matrix U.
  */
 void dPrintUblocks(int iam, int_t nsupers, gridinfo_t *grid,
