@@ -528,19 +528,20 @@ pdgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 	      routine. They will be freed after PDDISTRIBUTE routine.
 	      If options->Fact == SamePattern_SameRowPerm, these
 	      structures are not used.                                  */
-    fact_t   Fact;
-    double   *a;
-    int_t    *colptr, *rowind;
-    int_t    *perm_r; /* row permutations from partial pivoting */
-    int_t    *perm_c; /* column permutation vector */
-    int_t    *etree;  /* elimination tree */
-    int_t    *rowptr, *colind;  /* Local A in NR*/
-    int_t    colequ, Equil, factored, job, notran, rowequ, need_value;
-    int_t    i, iinfo, j, irow, m, n, nnz, permc_spec;
-    int_t    nnz_loc, m_loc, fst_row, icol;
-    int      iam,iam_g;
-    int      ldx;  /* LDA for matrix X (local). */
-    char     equed[1], norm[1];
+    fact_t  Fact;
+    double *a;
+    int_t   *colptr, *rowind;
+    int_t   *perm_r; /* row permutations from partial pivoting */
+    int_t   *perm_c; /* column permutation vector */
+    int_t   *etree;  /* elimination tree */
+    int_t   *rowptr, *colind;  /* Local A in NR*/
+    int_t   nnz_loc, nnz, iinfo;
+    int     m_loc, fst_row, icol;
+    int     colequ, Equil, factored, job, notran, rowequ, need_value;
+    int     i, j, irow, m, n, permc_spec;
+    int     iam, iam_g;
+    int     ldx;  /* LDA for matrix X (local). */
+    char    equed[1], norm[1];
     double   *C, *R, *C1, *R1, amax, anorm, colcnd, rowcnd;
     double   *X, *b_col, *b_work, *x_col;
     double   t;
@@ -606,9 +607,9 @@ pdgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 	*info = -5;
     else if ( nrhs < 0 )
 	*info = -6;
-    if ( sp_ienv_dist(2) > sp_ienv_dist(3) ) {
-        *info = 1;
-	printf("ERROR: Relaxation (NREL) cannot be larger than max. supernode size (NSUP).\n"
+    if ( sp_ienv_dist(2, options) > sp_ienv_dist(3, options) ) {
+        *info = -1;
+	printf("ERROR: Relaxation (SUPERLU_RELAX) cannot be larger than max. supernode size (SUPERLU_MAXSUP).\n"
 	"\t-> Check parameter setting in sp_ienv_dist.c to correct error.\n");
     }
     if ( *info ) {
@@ -719,11 +720,11 @@ pdgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 	    if ( iinfo > 0 ) {
 		if ( iinfo <= m ) {
 #if ( PRNTlevel>=1 )
-		    fprintf(stderr, "The " IFMT "-th row of A is exactly zero\n", iinfo);
+		    fprintf(stderr, "The %d-th row of A is exactly zero\n", (int)iinfo);
 #endif
 		} else {
 #if ( PRNTlevel>=1 )
-                    fprintf(stderr, "The " IFMT "-th column of A is exactly zero\n", iinfo-n);
+                    fprintf(stderr, "The %d-th column of A is exactly zero\n", (int)iinfo-n);
 #endif
                 }
  	    } else if ( iinfo < 0 ) return;
@@ -927,7 +928,7 @@ pdgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 	        stat->utime[ROWPERM] = t;
 #if ( PRNTlevel>=1 )
                 if ( !iam ) {
-		    printf(".. LDPERM job " IFMT "\t time: %.2f\n", job, t);
+		    printf(".. RowPerm %d\t time: %.2f\n", options->RowPerm, t);
 		    fflush(stdout);
 		}
 #endif
@@ -1061,7 +1062,7 @@ pdgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 #if ( PRNTlevel>=1 )
                 if ( !iam ) {
 		    printf(".. symbfact(): relax %d, maxsuper %d, fill %d\n",
-		          sp_ienv_dist(2), sp_ienv_dist(3), sp_ienv_dist(6));
+		          sp_ienv_dist(2,options), sp_ienv_dist(3,options), sp_ienv_dist(6,options));
 		    fflush(stdout);
 	        }
 #endif
@@ -1103,7 +1104,8 @@ pdgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 	    } /* end serial symbolic factorization */
 	    else {  /* parallel symbolic factorization */
 	    	t = SuperLU_timer_();
-	    	flinfo = symbfact_dist(nprocs_num, noDomains, A, perm_c, perm_r,
+	    	flinfo = symbfact_dist(options, nprocs_num, noDomains,
+		                       A, perm_c, perm_r,
 				       sizes, fstVtxSep, &Pslu_freeable,
 				       &(grid->comm), &symb_comm,
 				       &symb_mem_usage);
@@ -1141,7 +1143,7 @@ pdgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 	       NOTE: the row permutation Pc*Pr is applied internally in the
   	       distribution routine. */
 	    t = SuperLU_timer_();
-	    dist_mem_use = pddistribute(Fact, n, A, ScalePermstruct,
+	    dist_mem_use = pddistribute(options, n, A, ScalePermstruct,
                                       Glu_freeable, LUstruct, grid);
 	    stat->utime[DIST] = SuperLU_timer_() - t;
 
@@ -1158,7 +1160,7 @@ pdgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 	    for (j = 0; j < nnz_loc; ++j) colind[j] = perm_c[colind[j]];
 
     	    t = SuperLU_timer_();
-	    dist_mem_use = ddist_psymbtonum(Fact, n, A, ScalePermstruct,
+	    dist_mem_use = ddist_psymbtonum(options, n, A, ScalePermstruct,
 		  			   &Pslu_freeable, LUstruct, grid);
 	    if (dist_mem_use > 0)
 	        ABORT ("Not enough memory available for dist_psymbtonum\n");
@@ -1401,10 +1403,30 @@ pdgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 	       factorization with Fact == DOFACT or SamePattern is asked for. */
 	}
 
-	if ( options->DiagInv==YES &&
-             (options->SolveInitialized == NO || Fact == SamePattern ||
-              Fact == SamePattern_SameRowPerm) ) {
+#if ( defined(GPU_ACC) && defined(GPU_SOLVE) )
+        if(options->DiagInv==NO){
+	    if (iam==0) {
+	        printf("!!WARNING: GPU trisolve requires setting options->DiagInv==YES\n");
+                printf("           otherwise, use CPU trisolve\n");
+		fflush(stdout);
+	    }
+	    //exit(0);  // Sherry: need to return an error flag
+	}
+#endif
+
+	if ( options->DiagInv==YES && (Fact != FACTORED) ) {
 	    pdCompute_Diag_Inv(n, LUstruct, grid, stat, info);
+
+#ifdef GPU_ACC
+			pdconvertU(options, grid,LUstruct, stat, n);
+
+            checkGPU(gpuMemcpy(LUstruct->Llu->d_Linv_bc_dat, LUstruct->Llu->Linv_bc_dat,
+	        (LUstruct->Llu->Linv_bc_cnt) * sizeof(double), gpuMemcpyHostToDevice));
+            checkGPU(gpuMemcpy(LUstruct->Llu->d_Uinv_bc_dat, LUstruct->Llu->Uinv_bc_dat,
+	        (LUstruct->Llu->Uinv_bc_cnt) * sizeof(double), gpuMemcpyHostToDevice));
+            checkGPU(gpuMemcpy(LUstruct->Llu->d_Lnzval_bc_dat, LUstruct->Llu->Lnzval_bc_dat,
+	        (LUstruct->Llu->Lnzval_bc_cnt) * sizeof(double), gpuMemcpyHostToDevice));
+#endif
 	}
 
 
@@ -1412,7 +1434,7 @@ pdgssvx(superlu_dist_options_t *options, SuperMatrix *A,
     // {
 	// #pragma omp master
 	// {
-	pdgstrs(n, LUstruct, ScalePermstruct, grid, X, m_loc,
+	pdgstrs(options, n, LUstruct, ScalePermstruct, grid, X, m_loc,
 		fst_row, ldb, nrhs, SOLVEstruct, stat, info);
 	// }
 	// }
@@ -1494,7 +1516,7 @@ pdgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 			     Glu_persist, SOLVEstruct1);
 	    }
 
-	    pdgsrfs(n, A, anorm, LUstruct, ScalePermstruct, grid,
+	    pdgsrfs(options, n, A, anorm, LUstruct, ScalePermstruct, grid,
 		    B, ldb, X, ldx, nrhs, SOLVEstruct1, berr, stat, info);
 
             /* Deallocate the storage associated with SOLVEstruct1 */
@@ -1577,3 +1599,299 @@ pdgssvx(superlu_dist_options_t *options, SuperMatrix *A,
 #endif
 
 }
+
+
+
+
+void
+pdconvertU(superlu_dist_options_t *options, gridinfo_t *grid,
+	dLUstruct_t *LUstruct, SuperLUStat_t *stat, int_t n)
+{
+int64_t nnz_ind,nnz_offset;
+int64_t nnz_val;
+Glu_persist_t *Glu_persist = LUstruct->Glu_persist;
+int_t nsupers,nsupers_j,ncol,ncol_loc,nrow;
+int_t lk,ik,ub,nub,i,il,gik,k,uptr,jj,ii,fnz,irow,jb;
+dLocalLU_t *Llu = LUstruct->Llu;
+int_t  *Urbs = Llu->Urbs;
+int_t  **Ucb_valptr = Llu->Ucb_valptr;      /* Vertical linked list pointing to Unzval[] */
+Ucb_indptr_t **Ucb_indptr = Llu->Ucb_indptr;/* Vertical linked list pointing to Uindex[] */
+int_t knsupc,iknsupc,ikfrow,iklrow;
+int_t  *xsup = Glu_persist->xsup;; 
+
+int iam = grid->iam;
+int_t mycol = MYCOL (iam, grid);
+int_t myrow = MYROW (iam, grid);
+
+int_t  *usub;
+double *uval;
+
+int64_t Ucolind_bc_cnt=0;
+int64_t Unzval_bc_cnt=0, Unzval_br_cnt=0;
+int64_t Uindval_loc_bc_cnt=0;
+
+int_t next_lind;      /* next available position in index[*] */
+int_t next_lval;      /* next available position in nzval[*] */
+
+nsupers = Glu_persist->supno[n-1] + 1;
+nsupers_j = CEILING( nsupers, grid->npcol ); /* Number of local block columns */
+
+if ( !(Llu->Ucolind_bc_ptr = (int_t**)SUPERLU_MALLOC(nsupers_j * sizeof(int_t*))) )
+	ABORT("Malloc fails for Llu->Ucolind_bc_ptr[].");
+Llu->Ucolind_bc_ptr[nsupers_j-1] = NULL;
+
+if ( !(Llu->Unzval_bc_ptr =
+			(double**)SUPERLU_MALLOC(nsupers_j * sizeof(double*))) )
+	ABORT("Malloc fails for Llu->Unzval_bc_ptr[].");
+Llu->Unzval_bc_ptr[nsupers_j-1] = NULL;	
+
+if ( !(Llu->Uindval_loc_bc_ptr =
+			(int_t**)SUPERLU_MALLOC(nsupers_j * sizeof(int_t*))) )
+	ABORT("Malloc fails for Llu->Uindval_loc_bc_ptr[].");
+Llu->Uindval_loc_bc_ptr[nsupers_j-1] = NULL;
+
+if ( !(Llu->Uindval_loc_bc_offset =
+				(int64_t*)SUPERLU_MALLOC(nsupers_j * sizeof(int64_t))) ) {
+		fprintf(stderr, "Malloc fails for Llu->Uindval_loc_bc_offset[].");
+}
+Llu->Uindval_loc_bc_offset[nsupers_j-1] = -1;
+
+
+if ( !(Llu->Ucolind_bc_offset =
+			(int64_t*)SUPERLU_MALLOC(nsupers_j * sizeof(int64_t))) ) {
+	fprintf(stderr, "Malloc fails for Llu->Ucolind_bc_offset[].");
+}
+Llu->Ucolind_bc_offset[nsupers_j-1] = -1;	
+
+if ( !(Llu->Unzval_bc_offset =
+			(int64_t*)SUPERLU_MALLOC(nsupers_j * sizeof(int64_t))) ) {
+	fprintf(stderr, "Malloc fails for Llu->Lnzval_bc_offset[].");
+}
+Llu->Unzval_bc_offset[nsupers_j-1] = -1;		
+
+
+
+for (lk=0;lk<nsupers_j;++lk){
+	k = lk * grid->npcol + mycol;/* Global block number, col-wise. */	
+	knsupc = SuperSize( k );
+	nub = Urbs[lk];      /* Number of U blocks in block column lk */
+
+	if(nub>0){
+		// First pass count sizes of Llu->Ucolind_bc_ptr[lk] and Llu->Unzval_bc_ptr[lk]
+		nnz_ind=0;
+		nnz_val=0;
+		nnz_ind+=BC_HEADER_NEWU;	
+		nrow=0;
+		for (ub = 0; ub < nub; ++ub) {
+		ik = Ucb_indptr[lk][ub].lbnum; /* Local block number, row-wise. */
+		usub = Llu->Ufstnz_br_ptr[ik];
+		uval = Llu->Unzval_br_ptr[ik];
+		i = Ucb_indptr[lk][ub].indpos; /* Start of the block in usub[]. */
+		i += UB_DESCRIPTOR;
+		gik = ik * grid->nprow + myrow;/* Global block number, row-wise. */
+		iknsupc = SuperSize( gik );
+		nrow += iknsupc;
+		ikfrow = FstBlockC( gik );
+		iklrow = FstBlockC( gik+1 );
+		uptr = Ucb_valptr[lk][ub]; /* Start of the block in uval[]. */
+		
+		nnz_ind+=UB_DESCRIPTOR_NEWU;
+
+		for (jj = 0; jj < knsupc; ++jj) {
+		fnz = usub[i + jj];
+		if ( fnz < iklrow ) { /* Nonzero segment. */
+			nnz_val+=iknsupc;
+			nnz_ind+=1;
+			Unzval_br_cnt+=iklrow - fnz;
+			// for (irow = fnz; irow < iklrow; ++irow)
+			// dest[irow - ikfrow] -= uval[uptr++] * y[jj];
+			// stat->ops[SOLVE] += 2 * (iklrow - fnz);
+		}
+		} /* for jj ... */
+		} /* for ub ... */
+
+		// Second pass fills Llu->Ucolind_bc_ptr[lk] and Llu->Unzval_bc_ptr[lk]
+		if ( !(Llu->Ucolind_bc_ptr[lk] = intMalloc_dist(nnz_ind+nrow*2)) )
+			ABORT("Malloc fails for Llu->Ucolind_bc_ptr[lk]");
+		Llu->Ucolind_bc_offset[lk]=nnz_ind+nrow*2;
+		Ucolind_bc_cnt += Llu->Ucolind_bc_offset[lk];
+
+
+		if (!(Llu->Unzval_bc_ptr[lk]=doubleCalloc_dist(nnz_val)))
+	   	 	ABORT("Calloc fails for Llu->Unzval_bc_ptr[lk].");
+		Llu->Unzval_bc_offset[lk]=nnz_val;
+		Unzval_bc_cnt += Llu->Unzval_bc_offset[lk];
+
+
+		if ( !(Llu->Uindval_loc_bc_ptr[lk] = intCalloc_dist(nub*3)) )
+			ABORT("Malloc fails for Llu->Uindval_loc_bc_ptr[lk][]");
+		Llu->Uindval_loc_bc_offset[lk]=nub*3;
+		Uindval_loc_bc_cnt += Llu->Uindval_loc_bc_offset[lk];
+
+		Llu->Ucolind_bc_ptr[lk][0]=nub;
+		Llu->Ucolind_bc_ptr[lk][1]=nrow;
+		Llu->Ucolind_bc_ptr[lk][2]=nnz_ind;
+		nnz_offset=nnz_ind;
+
+		nnz_ind=0;
+		nnz_val=0;
+		ncol=0;
+		nnz_ind+=BC_HEADER_NEWU;	
+		nrow=0;
+		for (ub = 0; ub < nub; ++ub) {
+		ik = Ucb_indptr[lk][ub].lbnum; /* Local block number, row-wise. */
+		usub = Llu->Ufstnz_br_ptr[ik];
+		uval = Llu->Unzval_br_ptr[ik];
+		i = Ucb_indptr[lk][ub].indpos; /* Start of the block in usub[]. */
+		i += UB_DESCRIPTOR;
+		gik = ik * grid->nprow + myrow;/* Global block number, row-wise. */
+		iknsupc = SuperSize( gik );
+		ikfrow = FstBlockC( gik );
+		iklrow = FstBlockC( gik+1 );
+		uptr = Ucb_valptr[lk][ub]; /* Start of the block in uval[]. */
+		
+		for(ii=0; ii<iknsupc; ++ii){
+			Llu->Ucolind_bc_ptr[lk][nnz_offset+nrow*2] = ub;
+			Llu->Ucolind_bc_ptr[lk][nnz_offset+nrow*2+1] = ii;
+			nrow++;
+		}
+
+		ncol_loc=0;
+		for (jj = 0; jj < knsupc; ++jj) {
+		fnz = usub[i + jj];
+		if ( fnz < iklrow ) { /* Nonzero segment. */
+			Llu->Ucolind_bc_ptr[lk][nnz_ind+ncol_loc+UB_DESCRIPTOR_NEWU]=FstBlockC(k)+jj; /* Global column number */ 
+			ncol_loc++;
+			for (irow = fnz; irow < iklrow; ++irow){
+				Llu->Unzval_bc_ptr[lk][nnz_val+irow - ikfrow]=uval[uptr++];
+				// if(lk==2){
+				// printf("uval %5d %5d %5d %f %5d %5d \n",gik, uptr-1, irow - ikfrow, uval[uptr-1], Ucb_valptr[lk][ub],ub);
+				// // printf("Unzval_bc_ptr %5d %f\n",gik, Llu->Unzval_bc_ptr[lk][nnz_val+irow - ikfrow]);
+				// }
+			}
+			nnz_val+=iknsupc;
+		}
+		} /* for jj ... */
+		Llu->Ucolind_bc_ptr[lk][nnz_ind]=gik;
+		Llu->Ucolind_bc_ptr[lk][nnz_ind+1]=ncol_loc;
+
+		Llu->Uindval_loc_bc_ptr[lk][ub] = ik;
+		Llu->Uindval_loc_bc_ptr[lk][ub+nub] = nnz_ind;
+		Llu->Uindval_loc_bc_ptr[lk][ub+nub*2] = ncol;
+		// if(lk==69)
+		// 	printf("ub ncol_loc %5d %5d \n",ub, ncol_loc);
+		ncol+=ncol_loc*iknsupc;
+		nnz_ind+=ncol_loc+UB_DESCRIPTOR_NEWU; 
+		} /* for ub ... */
+
+	}else{
+		Llu->Ucolind_bc_ptr[lk] = NULL;
+		Llu->Unzval_bc_ptr[lk] = NULL;
+		Llu->Ucolind_bc_offset[lk]=-1;
+		Llu->Unzval_bc_offset[lk]=-1;
+		Llu->Uindval_loc_bc_ptr[lk] = NULL;
+		Llu->Uindval_loc_bc_offset[lk]=-1;
+	}
+}
+
+
+
+	// safe guard
+	Ucolind_bc_cnt +=1; 
+	Unzval_bc_cnt +=1; 
+	Uindval_loc_bc_cnt +=1; 
+	if ( !(Llu->Ucolind_bc_dat =
+				(int_t*)SUPERLU_MALLOC(Ucolind_bc_cnt * sizeof(int_t))) ) {
+		fprintf(stderr, "Malloc fails for Llu->Ucolind_bc_dat[].");
+	}		
+	if ( !(Llu->Unzval_bc_dat =
+				(double*)SUPERLU_MALLOC(Unzval_bc_cnt * sizeof(double))) ) {
+		fprintf(stderr, "Malloc fails for Llu->Unzval_bc_dat[].");
+	}	
+	if ( !(Llu->Uindval_loc_bc_dat =
+				(int_t*)SUPERLU_MALLOC(Uindval_loc_bc_cnt * sizeof(int_t))) ) {
+		fprintf(stderr, "Malloc fails for Llu->Uindval_loc_bc_dat[].");
+	}		
+
+	/* use contingous memory for Ucolind_bc_ptr, Unzval_bc_ptr, Uindval_loc_bc_ptr*/
+	k = CEILING( nsupers, grid->npcol );/* Number of local block columns */
+	Ucolind_bc_cnt=0;
+	Unzval_bc_cnt=0;
+	Uindval_loc_bc_cnt=0;
+	int64_t tmp_cnt;
+	
+	for (jb = 0; jb < k; ++jb) { /* for each block column ... */
+		if(Llu->Ucolind_bc_ptr[jb]!=NULL){
+			for (jj = 0; jj < Llu->Ucolind_bc_offset[jb]; ++jj) {
+				Llu->Ucolind_bc_dat[Ucolind_bc_cnt+jj]=Llu->Ucolind_bc_ptr[jb][jj];
+			}
+			SUPERLU_FREE(Llu->Ucolind_bc_ptr[jb]);
+			Llu->Ucolind_bc_ptr[jb]=&Llu->Ucolind_bc_dat[Ucolind_bc_cnt];
+			tmp_cnt = Llu->Ucolind_bc_offset[jb];
+			Llu->Ucolind_bc_offset[jb]=Ucolind_bc_cnt;
+			Ucolind_bc_cnt+=tmp_cnt;
+		}
+
+		if(Llu->Unzval_bc_ptr[jb]!=NULL){
+			for (jj = 0; jj < Llu->Unzval_bc_offset[jb]; ++jj) {
+				Llu->Unzval_bc_dat[Unzval_bc_cnt+jj]=Llu->Unzval_bc_ptr[jb][jj];
+			}
+			SUPERLU_FREE(Llu->Unzval_bc_ptr[jb]);
+			Llu->Unzval_bc_ptr[jb]=&Llu->Unzval_bc_dat[Unzval_bc_cnt];
+			tmp_cnt = Llu->Unzval_bc_offset[jb];
+			Llu->Unzval_bc_offset[jb]=Unzval_bc_cnt;
+			Unzval_bc_cnt+=tmp_cnt;
+		}
+
+		if(Llu->Uindval_loc_bc_ptr[jb]!=NULL){
+			for (jj = 0; jj < Llu->Uindval_loc_bc_offset[jb]; ++jj) {
+				Llu->Uindval_loc_bc_dat[Uindval_loc_bc_cnt+jj]=Llu->Uindval_loc_bc_ptr[jb][jj];
+			}
+			SUPERLU_FREE(Llu->Uindval_loc_bc_ptr[jb]);
+			Llu->Uindval_loc_bc_ptr[jb]=&Llu->Uindval_loc_bc_dat[Uindval_loc_bc_cnt];
+			tmp_cnt = Llu->Uindval_loc_bc_offset[jb];
+			Llu->Uindval_loc_bc_offset[jb]=Uindval_loc_bc_cnt;
+			Uindval_loc_bc_cnt+=tmp_cnt;
+		}	
+
+
+	}	
+	Llu->Ucolind_bc_cnt = Ucolind_bc_cnt;
+	Llu->Unzval_bc_cnt = Unzval_bc_cnt;
+	Llu->Uindval_loc_bc_cnt = Uindval_loc_bc_cnt;
+	// printf("Ucolind_bc_cnt %10d\n",Ucolind_bc_cnt);
+	printf("Unzval_bc_cnt %10ld v.s. Unzval_br_cnt %10ld\n",Unzval_bc_cnt,Unzval_br_cnt);
+	// printf("Llu->Ucolind_bc_offset %10d\n",Llu->Ucolind_bc_offset[0]);
+
+#ifdef GPU_ACC
+	checkGPU(gpuFree(Llu->d_Ucolind_bc_dat));
+	checkGPU(gpuFree(Llu->d_Ucolind_bc_offset));
+	checkGPU(gpuFree(Llu->d_Unzval_bc_dat));
+	checkGPU(gpuFree(Llu->d_Unzval_bc_offset));
+	checkGPU(gpuFree(Llu->d_Uindval_loc_bc_dat));
+	checkGPU(gpuFree(Llu->d_Uindval_loc_bc_offset));
+
+
+	checkGPU(gpuMalloc( (void**)&Llu->d_Ucolind_bc_dat, (Llu->Ucolind_bc_cnt) * sizeof(int_t)));
+	checkGPU(gpuMemcpy(Llu->d_Ucolind_bc_dat, Llu->Ucolind_bc_dat, (Llu->Ucolind_bc_cnt) * sizeof(int_t), gpuMemcpyHostToDevice));	
+	checkGPU(gpuMalloc( (void**)&Llu->d_Ucolind_bc_offset, CEILING( nsupers, grid->npcol ) * sizeof(int64_t)));
+	checkGPU(gpuMemcpy(Llu->d_Ucolind_bc_offset, Llu->Ucolind_bc_offset, CEILING( nsupers, grid->npcol ) * sizeof(int64_t), gpuMemcpyHostToDevice));	
+	checkGPU(gpuMalloc( (void**)&Llu->d_Unzval_bc_offset, CEILING( nsupers, grid->npcol ) * sizeof(int64_t)));
+	checkGPU(gpuMemcpy(Llu->d_Unzval_bc_offset, Llu->Unzval_bc_offset, CEILING( nsupers, grid->npcol ) * sizeof(int64_t), gpuMemcpyHostToDevice));	
+	checkGPU(gpuMalloc( (void**)&Llu->d_Unzval_bc_dat, (Llu->Unzval_bc_cnt) * sizeof(double)));
+	checkGPU(gpuMemcpy(LUstruct->Llu->d_Unzval_bc_dat, LUstruct->Llu->Unzval_bc_dat,(LUstruct->Llu->Unzval_bc_cnt) * sizeof(double), gpuMemcpyHostToDevice));
+	
+	checkGPU(gpuMalloc( (void**)&Llu->d_Uindval_loc_bc_dat, (Llu->Uindval_loc_bc_cnt) * sizeof(int_t)));
+	checkGPU(gpuMemcpy(Llu->d_Uindval_loc_bc_dat, Llu->Uindval_loc_bc_dat, (Llu->Uindval_loc_bc_cnt) * sizeof(int_t), gpuMemcpyHostToDevice));		
+	checkGPU(gpuMalloc( (void**)&Llu->d_Uindval_loc_bc_offset, CEILING( nsupers, grid->npcol ) * sizeof(int64_t)));
+	checkGPU(gpuMemcpy(Llu->d_Uindval_loc_bc_offset, Llu->Uindval_loc_bc_offset, CEILING( nsupers, grid->npcol ) * sizeof(int64_t), gpuMemcpyHostToDevice));	
+#endif
+
+	SUPERLU_FREE (Llu->Ucolind_bc_dat);
+	SUPERLU_FREE (Llu->Ucolind_bc_offset);
+	SUPERLU_FREE (Llu->Unzval_bc_dat);
+	SUPERLU_FREE (Llu->Unzval_bc_offset);
+	SUPERLU_FREE (Llu->Uindval_loc_bc_dat);
+	SUPERLU_FREE (Llu->Uindval_loc_bc_offset);
+
+}
