@@ -481,7 +481,7 @@ at the top-level directory.
  *
  * stat   (output) SuperLUStat_t*
  *        Record the statistics on runtime and floating-point operation count.
- *        See util.h for the definition of 'SuperLUStat_t'.
+ *        See util_dist.h for the definition of 'SuperLUStat_t'.
  *
  * info    (output) int*
  *         = 0: successful exit
@@ -1007,7 +1007,7 @@ pdgssvx3d (superlu_dist_options_t * options, SuperMatrix * A,
 	
 	
 	/* ------------------------------------------------------------
-	   Perform the LU factorization.
+	   Perform ordering and symbolic factorization
 	   ------------------------------------------------------------ */
 	if ( !factored ) {
 	    t = SuperLU_timer_ ();
@@ -1106,7 +1106,7 @@ pdgssvx3d (superlu_dist_options_t * options, SuperMatrix * A,
 		    if (!iam)
 			printf
 			    (".. symbfact(): relax %4d, maxsuper %4d, fill %4d\n",
-			     sp_ienv_dist(2), sp_ienv_dist(3), sp_ienv_dist(6));
+			     sp_ienv_dist(2,options), sp_ienv_dist(3,options), sp_ienv_dist(6,options));
 #endif
 		    t = SuperLU_timer_ ();
 		    if (!(Glu_freeable = (Glu_freeable_t *)
@@ -1150,7 +1150,8 @@ pdgssvx3d (superlu_dist_options_t * options, SuperMatrix * A,
 		else { /* parallel symbolic factorization */
 		    t = SuperLU_timer_ ();
 		    flinfo =
-			symbfact_dist (nprocs_num, noDomains, A, perm_c, perm_r,
+			symbfact_dist (options, nprocs_num, noDomains,
+			               A, perm_c, perm_r,
 				       sizes, fstVtxSep, &Pslu_freeable,
 				       &(grid->comm), &symb_comm,
 				       &symb_mem_usage);
@@ -1184,8 +1185,9 @@ pdgssvx3d (superlu_dist_options_t * options, SuperMatrix * A,
 		   NOTE: the row permutation Pc*Pr is applied internally in the
 		   distribution routine. */
 		t = SuperLU_timer_ ();
-		dist_mem_use = pddistribute (Fact, n, A, ScalePermstruct,
+		dist_mem_use = pddistribute (options, n, A, ScalePermstruct,
 					     Glu_freeable, LUstruct, grid);
+
 		stat->utime[DIST] = SuperLU_timer_ () - t;
 		
 		/* Deallocate storage used in symbolic factorization. */
@@ -1203,7 +1205,7 @@ pdgssvx3d (superlu_dist_options_t * options, SuperMatrix * A,
 		    colind[j] = perm_c[colind[j]];
 		
 		t = SuperLU_timer_ ();
-		dist_mem_use = ddist_psymbtonum (Fact, n, A, ScalePermstruct,
+		dist_mem_use = ddist_psymbtonum (options, n, A, ScalePermstruct,
 						 &Pslu_freeable, LUstruct, grid);
 		if (dist_mem_use > 0)
 		    ABORT ("Not enough memory available for dist_psymbtonum\n");
@@ -1213,9 +1215,10 @@ pdgssvx3d (superlu_dist_options_t * options, SuperMatrix * A,
 
 	    /*if (!iam) printf ("\tDISTRIBUTE time  %8.2f\n", stat->utime[DIST]); */
 	} /* end if not Factored */
-    } /* end if process layer 0 */
+	
+    } /* end 2D process layer 0 */
 
-    trf3Dpartition_t*  trf3Dpartition;
+    dtrf3Dpartition_t*  trf3Dpartition;
 
     /* Perform numerical factorization in parallel on all process layers.*/
     if ( !factored ) {
@@ -1240,6 +1243,7 @@ pdgssvx3d (superlu_dist_options_t * options, SuperMatrix * A,
 	}
 #endif
 
+
 	t = SuperLU_timer_ ();
 
 	/*factorize in grid 1*/
@@ -1255,7 +1259,7 @@ pdgssvx3d (superlu_dist_options_t * options, SuperMatrix * A,
 
 	SCT->gatherLUtimer += SuperLU_timer_() - tgather;
 	/*print stats for bottom grid*/
-	
+
 #if ( PRNTlevel>=1 )
 	if (!grid3d->zscp.Iam)
 	    {
@@ -1382,7 +1386,7 @@ pdgssvx3d (superlu_dist_options_t * options, SuperMatrix * A,
 		/* ------------------------------------------------------
 		   Solve the linear system.
 		   ------------------------------------------------------*/
-		if (options->SolveInitialized == NO) /* First time */
+		if (options->SolveInitialized == NO) { /* First time */
                    /* Inside this routine, SolveInitialized is set to YES.
 	              For repeated call to pdgssvx3d(), no need to re-initialilze
 	              the Solve data & communication structures, unless a new
@@ -1391,12 +1395,98 @@ pdgssvx3d (superlu_dist_options_t * options, SuperMatrix * A,
 			dSolveInit (options, A, perm_r, perm_c, nrhs, LUstruct,
 			            grid, SOLVEstruct);
 		    }
+		}
+#if ( defined(GPU_ACC) && defined(GPU_SOLVE) )
+        if(options->DiagInv==NO){
+	    if (iam==0) {
+	        printf("!!WARNING: GPU trisolve requires setting options->DiagInv==YES\n");
+                printf("           otherwise, use CPU trisolve\n");
+		fflush(stdout);
+	    }
+	    //exit(0);  // Sherry: need to return an error flag
+	}
+#endif
+
+	if ( options->DiagInv==YES && (Fact != FACTORED) ) {
+	    pdCompute_Diag_Inv(n, LUstruct, grid, stat, info);
+
+
+
+// The following #ifdef GPU_ACC block frees and reallocates GPU data for trisolve. The data seems to be overwritten by pdgstrf3d. 
+int_t nsupers = getNsupers(n, LUstruct->Glu_persist);
+#ifdef GPU_ACC
+	
+	pdconvertU(options, grid,LUstruct, stat, n);
+
+	checkGPU(gpuFree(LUstruct->Llu->d_xsup));
+	checkGPU(gpuFree(LUstruct->Llu->d_LRtree_ptr));
+	checkGPU(gpuFree(LUstruct->Llu->d_LBtree_ptr));
+	checkGPU(gpuFree(LUstruct->Llu->d_URtree_ptr));
+	checkGPU(gpuFree(LUstruct->Llu->d_UBtree_ptr));	
+	checkGPU(gpuFree(LUstruct->Llu->d_Lrowind_bc_dat));
+	checkGPU(gpuFree(LUstruct->Llu->d_Lindval_loc_bc_dat));
+	checkGPU(gpuFree(LUstruct->Llu->d_Lrowind_bc_offset));
+	checkGPU(gpuFree(LUstruct->Llu->d_Lindval_loc_bc_offset));
+	checkGPU(gpuFree(LUstruct->Llu->d_Lnzval_bc_offset));
+	checkGPU(gpuFree(LUstruct->Llu->d_Linv_bc_offset));
+	checkGPU(gpuFree(LUstruct->Llu->d_Uinv_bc_offset));
+	checkGPU(gpuFree(LUstruct->Llu->d_ilsum));
+	checkGPU(gpuFree(LUstruct->Llu->d_Lnzval_bc_dat));
+	checkGPU(gpuFree(LUstruct->Llu->d_Linv_bc_dat));
+	checkGPU(gpuFree(LUstruct->Llu->d_Uinv_bc_dat));
+
+
+
+	checkGPU(gpuMalloc( (void**)&LUstruct->Llu->d_xsup, (n+1) * sizeof(int_t)));
+	checkGPU(gpuMemcpy(LUstruct->Llu->d_xsup, LUstruct->Glu_persist->xsup, (n+1) * sizeof(int_t), gpuMemcpyHostToDevice));
+	checkGPU(gpuMalloc( (void**)&LUstruct->Llu->d_LRtree_ptr, CEILING( nsupers, grid->nprow ) * sizeof(C_Tree)));
+	checkGPU(gpuMalloc( (void**)&LUstruct->Llu->d_LBtree_ptr, CEILING( nsupers, grid->npcol ) * sizeof(C_Tree)));
+	checkGPU(gpuMalloc( (void**)&LUstruct->Llu->d_URtree_ptr, CEILING( nsupers, grid->nprow ) * sizeof(C_Tree)));
+	checkGPU(gpuMalloc( (void**)&LUstruct->Llu->d_UBtree_ptr, CEILING( nsupers, grid->npcol ) * sizeof(C_Tree)));	
+	checkGPU(gpuMemcpy(LUstruct->Llu->d_LRtree_ptr, LUstruct->Llu->LRtree_ptr, CEILING( nsupers, grid->nprow ) * sizeof(C_Tree), gpuMemcpyHostToDevice));	
+	checkGPU(gpuMemcpy(LUstruct->Llu->d_LBtree_ptr, LUstruct->Llu->LBtree_ptr, CEILING( nsupers, grid->npcol ) * sizeof(C_Tree), gpuMemcpyHostToDevice));			
+	checkGPU(gpuMemcpy(LUstruct->Llu->d_URtree_ptr, LUstruct->Llu->URtree_ptr, CEILING( nsupers, grid->nprow ) * sizeof(C_Tree), gpuMemcpyHostToDevice));	
+	checkGPU(gpuMemcpy(LUstruct->Llu->d_UBtree_ptr, LUstruct->Llu->UBtree_ptr, CEILING( nsupers, grid->npcol ) * sizeof(C_Tree), gpuMemcpyHostToDevice));		
+	checkGPU(gpuMalloc( (void**)&LUstruct->Llu->d_Lrowind_bc_dat, (LUstruct->Llu->Lrowind_bc_cnt) * sizeof(int_t)));
+	checkGPU(gpuMemcpy(LUstruct->Llu->d_Lrowind_bc_dat, LUstruct->Llu->Lrowind_bc_dat, (LUstruct->Llu->Lrowind_bc_cnt) * sizeof(int_t), gpuMemcpyHostToDevice));	
+	checkGPU(gpuMalloc( (void**)&LUstruct->Llu->d_Lindval_loc_bc_dat, (LUstruct->Llu->Lindval_loc_bc_cnt) * sizeof(int_t)));
+	checkGPU(gpuMemcpy(LUstruct->Llu->d_Lindval_loc_bc_dat, LUstruct->Llu->Lindval_loc_bc_dat, (LUstruct->Llu->Lindval_loc_bc_cnt) * sizeof(int_t), gpuMemcpyHostToDevice));	
+	checkGPU(gpuMalloc( (void**)&LUstruct->Llu->d_Lrowind_bc_offset, CEILING( nsupers, grid->npcol ) * sizeof(long int)));
+	checkGPU(gpuMemcpy(LUstruct->Llu->d_Lrowind_bc_offset, LUstruct->Llu->Lrowind_bc_offset, CEILING( nsupers, grid->npcol ) * sizeof(long int), gpuMemcpyHostToDevice));	
+	checkGPU(gpuMalloc( (void**)&LUstruct->Llu->d_Lindval_loc_bc_offset, CEILING( nsupers, grid->npcol ) * sizeof(long int)));
+	checkGPU(gpuMemcpy(LUstruct->Llu->d_Lindval_loc_bc_offset, LUstruct->Llu->Lindval_loc_bc_offset, CEILING( nsupers, grid->npcol ) * sizeof(long int), gpuMemcpyHostToDevice));	
+	checkGPU(gpuMalloc( (void**)&LUstruct->Llu->d_Lnzval_bc_offset, CEILING( nsupers, grid->npcol ) * sizeof(long int)));
+	checkGPU(gpuMemcpy(LUstruct->Llu->d_Lnzval_bc_offset, LUstruct->Llu->Lnzval_bc_offset, CEILING( nsupers, grid->npcol ) * sizeof(long int), gpuMemcpyHostToDevice));	
+	checkGPU(gpuMalloc( (void**)&LUstruct->Llu->d_Linv_bc_offset, CEILING( nsupers, grid->npcol ) * sizeof(long int)));
+	checkGPU(gpuMemcpy(LUstruct->Llu->d_Linv_bc_offset, LUstruct->Llu->Linv_bc_offset, CEILING( nsupers, grid->npcol ) * sizeof(long int), gpuMemcpyHostToDevice));	
+	checkGPU(gpuMalloc( (void**)&LUstruct->Llu->d_Uinv_bc_offset, CEILING( nsupers, grid->npcol ) * sizeof(long int)));
+	checkGPU(gpuMemcpy(LUstruct->Llu->d_Uinv_bc_offset, LUstruct->Llu->Uinv_bc_offset, CEILING( nsupers, grid->npcol ) * sizeof(long int), gpuMemcpyHostToDevice));		
+	checkGPU(gpuMalloc( (void**)&LUstruct->Llu->d_ilsum, (CEILING( nsupers, grid->nprow )+1) * sizeof(int_t)));
+	checkGPU(gpuMemcpy(LUstruct->Llu->d_ilsum, LUstruct->Llu->ilsum, (CEILING( nsupers, grid->nprow )+1) * sizeof(int_t), gpuMemcpyHostToDevice));
+	checkGPU(gpuMalloc( (void**)&LUstruct->Llu->d_Lnzval_bc_dat, (LUstruct->Llu->Lnzval_bc_cnt) * sizeof(double)));
+	checkGPU(gpuMalloc( (void**)&LUstruct->Llu->d_Linv_bc_dat, (LUstruct->Llu->Linv_bc_cnt) * sizeof(double)));
+	checkGPU(gpuMalloc( (void**)&LUstruct->Llu->d_Uinv_bc_dat, (LUstruct->Llu->Uinv_bc_cnt) * sizeof(double)));
+#endif	
+
+
+#ifdef GPU_ACC
+
+	checkGPU(gpuMemcpy(LUstruct->Llu->d_Linv_bc_dat, LUstruct->Llu->Linv_bc_dat,
+	(LUstruct->Llu->Linv_bc_cnt) * sizeof(double), gpuMemcpyHostToDevice));
+	checkGPU(gpuMemcpy(LUstruct->Llu->d_Uinv_bc_dat, LUstruct->Llu->Uinv_bc_dat,
+	(LUstruct->Llu->Uinv_bc_cnt) * sizeof(double), gpuMemcpyHostToDevice));
+	checkGPU(gpuMemcpy(LUstruct->Llu->d_Lnzval_bc_dat, LUstruct->Llu->Lnzval_bc_dat,
+	(LUstruct->Llu->Lnzval_bc_cnt) * sizeof(double), gpuMemcpyHostToDevice));
+#endif
+	}
+
+
 		stat->utime[SOLVE] = 0.0;
 #if 0 // Sherry: the following interface is needed by 3D trisolve.
 		pdgstrs_vecpar (n, LUstruct, ScalePermstruct, grid, X, m_loc,
 				fst_row, ldb, nrhs, SOLVEstruct, stat, info);
 #else
-		pdgstrs(n, LUstruct, ScalePermstruct, grid, X, m_loc,
+		pdgstrs(options, n, LUstruct, ScalePermstruct, grid, X, m_loc,
 			fst_row, ldb, nrhs, SOLVEstruct, stat, info);
 #endif
 
@@ -1484,7 +1574,7 @@ pdgssvx3d (superlu_dist_options_t * options, SuperMatrix * A,
 					      Glu_persist, SOLVEstruct1);
 			    }
 			
-			pdgsrfs (n, A, anorm, LUstruct, ScalePermstruct, grid,
+			pdgsrfs(options, n, A, anorm, LUstruct, ScalePermstruct, grid,
 				 B, ldb, X, ldx, nrhs, SOLVEstruct1, berr, stat, info);
 			
 			/* Deallocate the storage associated with SOLVEstruct1 */
