@@ -27,6 +27,8 @@ at the top-level directory.
  *     October 5, 2021     version 7.1.0
  *     October 18, 2021    version 7.1.1
  *     December 12, 2021   version 7.2.0
+ *     May 22, 2022        version 8.0.0
+ *     July 5, 2022        version 8.1.0
  * </pre>
  */
 
@@ -45,11 +47,13 @@ at the top-level directory.
    #include <omp.h>
 #endif
 
+
 #include <mpi.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <limits.h>
 #include <string.h>
+#include <ctype.h>
 // #include <stdatomic.h>
 #include <math.h>
 #include <stdint.h>
@@ -76,12 +80,29 @@ at the top-level directory.
  *   #endif
  * Versions 4.x and earlier do not include a #define'd version numbers.
  */
-#define SUPERLU_DIST_MAJOR_VERSION     7
-#define SUPERLU_DIST_MINOR_VERSION     2
+#define SUPERLU_DIST_MAJOR_VERSION     8
+#define SUPERLU_DIST_MINOR_VERSION     0
 #define SUPERLU_DIST_PATCH_VERSION     0
-#define SUPERLU_DIST_RELEASE_DATE      "December 12, 2021"
+#define SUPERLU_DIST_RELEASE_DATE      "May 22, 2022"
 
 #include "superlu_dist_config.h"
+
+#ifdef HAVE_CUDA
+#define GPU_ACC
+//#include "cublas_utils.h"
+#endif
+
+#ifdef HAVE_HIP
+#ifndef GPU_ACC
+#define GPU_ACC
+#endif
+#endif
+
+#ifdef GPU_ACC
+#include "gpu_api_utils.h"
+#endif
+
+
 /* Define my integer size int_t */
 #ifdef _CRAY
   typedef short int_t;
@@ -97,27 +118,6 @@ at the top-level directory.
   #define IFMT "%8d"
 #endif
 
-//#ifdef __INTEL_COMPILER
-//#include "mkl.h"
-//#endif
-
-#if 0 // Sherry: the following does not work with gcc on Linux.
-#define  _mm_malloc(a,b) malloc(a)
-#define _mm_free(a) free(a)
-
-static __inline__ unsigned long long _rdtsc(void)
-{
-    unsigned long long int x;
-    //    __asm__ volatile (".byte 0x0f, 0x31" : "=A" (x));
-    x = 0;
-    return x;
-}
-#endif
-
-#ifdef HAVE_CUDA  
-#define GPU_ACC   // enable CUDA
-#include "cublas_utils.h"
-#endif
 
 /* MPI C complex datatype */
 #define SuperLU_MPI_COMPLEX         MPI_C_COMPLEX 
@@ -135,10 +135,6 @@ typedef MPI_C_DOUBLE_COMPLEX  SuperLU_MPI_DOUBLE_COMPLEX;
 #include "util_dist.h"
 #include "psymbfact.h"
 
-#ifdef GPU_ACC
-#include <cuda.h>
-#endif
-
 
 #define MAX_SUPER_SIZE 512   /* Sherry: moved from superlu_gpu.cu */
 
@@ -197,6 +193,8 @@ typedef MPI_C_DOUBLE_COMPLEX  SuperLU_MPI_DOUBLE_COMPLEX;
 #define LB_DESCRIPTOR  2
 #define BR_HEADER      3
 #define UB_DESCRIPTOR  2
+#define BC_HEADER_NEWU      3
+#define UB_DESCRIPTOR_NEWU  2
 #define NBUFFERS       5
 
 /*
@@ -302,6 +300,42 @@ static const int RD_U=4;	/* MPI tag for lsum in U-solve*/
 #endif /* MSVC */
 #endif /* SUPERLU_DIST_EXPORT */
 
+
+/*
+ * CONSTANTS in MAGMA
+ */
+#ifndef MAGMA_CONST
+#define MAGMA_CONST
+
+// #define DIM_X  32
+// #define DIM_Y  16
+
+#define DIM_X  16
+#define DIM_Y  16
+
+
+#define BLK_M  DIM_X*4
+#define BLK_N  DIM_Y*4
+#define BLK_K 2048/(BLK_M)
+
+#define DIM_XA  DIM_X
+#define DIM_YA  DIM_Y
+#define DIM_XB  DIM_X
+#define DIM_YB  DIM_Y
+
+#define NWARP  DIM_X*DIM_Y/32
+
+// // // // // // #define TILE_SIZE  32
+
+
+#define THR_M ( BLK_M / DIM_X )
+#define THR_N ( BLK_N / DIM_Y )
+
+#define fetch(A, m, n, bound) offs_d##A[min(n*LD##A+m, bound)]
+#define fma(A, B, C) C += (A*B)
+#endif
+/*---- end MAGMA ----*/
+    
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -380,11 +414,11 @@ typedef struct {
 			   *                     0      3      6      9
 			   *                     1      4      7      10      
 			   *                     2      5      8      11
-			   * = 1: XY-major (need set env. var.: RANKORDER=XY)
+			   * = 1: XY-major (need set env. var.: SUPERLU_RANKORDER=XY)
 			   *    e.g. 1x3x4 grid: layer0 layer1 layer2 layer3
-			   *                     0      1      2      4
-			   *                     5      6      7      8
-			   *                     9      10     11     12
+			   *                     0      1      2      3
+			   *                     4      5      6      7
+			   *                     8      9     10     11
 			   */
 } gridinfo3d_t;
 
@@ -704,10 +738,19 @@ typedef struct {
     yes_no_t      RefineInitialized;
     yes_no_t      PrintStat;
     //int           nnzL, nnzU;      /* used to store nnzs for now       */
-    int           num_lookaheads;  /* num of levels in look-ahead      */
     yes_no_t      lookahead_etree; /* use etree computed from the
 				      serial symbolic factorization */
+    int num_lookaheads;  /* num of levels in look-ahead      */
+    int superlu_relax;   /* max. allowed relaxed supernode size; see sp_ienv(2) */
+    int superlu_maxsup;  /* max. allowed supernode size; see sp_ienv(3) */
+    char superlu_rankorder[4]; /* Z-major or XY-majir order in 3D grid */
+    char superlu_lbs[4]; /* etree load balancing strategy in 3D algorithm */
+    int superlu_n_gemm; /* one of GEMM offload criteria; see sp_ienv(7) */
+    int superlu_max_buffer_size; /* max. buffer size on GPU; see sp_ienv(8) */
+    int superlu_num_gpu_streams; /* number of GPU streams; see sp_ienv(9) */
+    int superlu_acc_offload; /* whether to offload work to GPU; see sp_ienv(10) */
     yes_no_t      SymPattern;      /* symmetric factorization          */
+    yes_no_t      Use_TensorCore;  /* Use Tensor Core or not  */
     yes_no_t      Algo3d;          /* use 3D factorization/solve algorithms */
 } superlu_dist_options_t;
 
@@ -725,7 +768,7 @@ typedef struct {
 } Ucb_indptr_t;
 
 /* 
- *-- The new structures added in the hybrid CUDA + OpenMP + MPI code.
+ *-- The new structures added in the hybrid GPU + OpenMP + MPI code.
  */
 typedef struct {
     int_t rukp;
@@ -762,6 +805,65 @@ struct superlu_pair
 
 /*==== For 3D code ====*/
 
+typedef struct
+{
+  int_t nub;
+  int_t klst;
+  int_t ldu;
+  int_t* usub;
+  //double *uval;
+} uPanelInfo_t;
+
+typedef struct
+{
+  int_t *lsub;
+  //double *lusup;
+  void *lusup;
+  int_t luptr0;
+  int_t nlb;  //number of l blocks                                                      
+  int_t nsupr;
+} lPanelInfo_t;
+
+typedef struct
+{
+    Ublock_info_t* Ublock_info;
+    Remain_info_t*  Remain_info;
+    uPanelInfo_t* uPanelInfo;
+    lPanelInfo_t* lPanelInfo;
+} packLUInfo_t;
+
+
+/* HyP_t is the data structure to assist HALO offload of Schur-complement. */
+typedef struct
+{
+  Remain_info_t *lookAhead_info, *Remain_info;
+  Ublock_info_t *Ublock_info, *Ublock_info_Phi;
+
+  int_t first_l_block_acc , first_u_block_acc;
+  int_t last_offload ;
+  int_t *Lblock_dirty_bit, * Ublock_dirty_bit;
+  void *lookAhead_L_buff, *Remain_L_buff;
+  int_t lookAheadBlk;  /* number of blocks in look-ahead window */
+  int_t RemainBlk ;    /* number of blocks outside look-ahead window */
+  int_t  num_look_aheads, nsupers;
+  int_t ldu, ldu_Phi;
+  int_t num_u_blks, num_u_blks_Phi;
+
+  int_t jj_cpu;
+  void *bigU_Phi;
+  void *bigU_host;
+  int_t Lnbrow;
+  int_t Rnbrow;
+
+  int_t buffer_size;
+  int_t bigu_size;
+  int offloadCondition;
+  int superlu_acc_offload;
+  int nGPUStreams;
+} HyP_t;
+
+
+
 /* return the mpi_tag assuming 5 pairs of communications and MPI_TAG_UB >= 5 *
  * for each supernodal column, the five communications are:                  *
  * 0,1: for sending L to "right"                                             *
@@ -797,7 +899,7 @@ typedef struct
 
 
 //global variable 
-extern double CPU_CLOCK_RATE;
+// extern double CPU_CLOCK_RATE;
 
 typedef struct
 {
@@ -910,7 +1012,6 @@ typedef struct
     int_t *perm_u;
     int *indirect;
     int *indirect2;
-    
 } factNodelists_t;
 
 typedef struct
@@ -946,7 +1047,7 @@ typedef struct xtrsTimer_t
     double ppXmem;		// perprocess X-memory
 } xtrsTimer_t;
 
-/*==== For 3D code ====*/
+/*==== end For 3D code ====*/
 
 /*====================*/
 
@@ -988,7 +1089,8 @@ extern void  bcast_tree(void *, int, MPI_Datatype, int, int,
 			gridinfo_t *, int, int *);
 extern int_t symbfact(superlu_dist_options_t *, int, SuperMatrix *, int_t *,
                       int_t *, Glu_persist_t *, Glu_freeable_t *);
-extern int_t symbfact_SubInit(fact_t, void *, int_t, int_t, int_t, int_t,
+extern int_t symbfact_SubInit(superlu_dist_options_t *options,
+			      fact_t, void *, int_t, int_t, int_t, int_t,
 			      Glu_persist_t *, Glu_freeable_t *);
 extern int_t symbfact_SubXpand(int_t, int_t, int_t, MemType, int_t *,
 			       Glu_freeable_t *);
@@ -1002,9 +1104,11 @@ extern float   smach_dist(char *);
 extern double  dmach_dist(char *);
 extern void    *superlu_malloc_dist (size_t);
 extern void    superlu_free_dist (void*);
+extern int   *int32Malloc_dist (int);
+extern int   *int32Calloc_dist (int);
 extern int_t   *intMalloc_dist (int_t);
 extern int_t   *intCalloc_dist (int_t);
-extern int_t   mc64id_dist(int_t *);
+extern int   mc64id_dist(int *);
 extern void  arrive_at_ublock (int_t, int_t *, int_t *, int_t *,
 			       int_t *, int_t *, int_t, int_t, 
 			       int_t *, int_t *, int_t *, gridinfo_t *);
@@ -1014,7 +1118,7 @@ extern int_t estimate_bigu_size (int_t, int_t **, Glu_persist_t *,
 /* Auxiliary routines */
 extern double SuperLU_timer_ ();
 extern void   superlu_abort_and_exit_dist(char *);
-extern int    sp_ienv_dist (int);
+extern int    sp_ienv_dist (int, superlu_dist_options_t *);
 extern void   ifill_dist (int_t *, int_t, int_t);
 extern void   super_stats_dist (int_t, int_t *);
 extern void  get_diag_procs(int_t, Glu_persist_t *, gridinfo_t *, int_t *,
@@ -1035,7 +1139,8 @@ extern int_t partitionM( int_t*, int_t, int_t, int_t, int_t, int_t);
 
 /* Prototypes for parallel symbolic factorization */
 extern float symbfact_dist
-(int,  int, SuperMatrix *, int_t *, int_t *,  int_t *, int_t *,
+(superlu_dist_options_t *, int,  int,
+ SuperMatrix *, int_t *, int_t *,  int_t *, int_t *,
  Pslu_freeable_t *, MPI_Comm *, MPI_Comm *,  superlu_dist_mem_usage_t *);
 
 /* Get the column permutation using parmetis */
@@ -1047,7 +1152,7 @@ extern float get_perm_c_parmetis
    the parallel symbolic factorization routine */
 
 extern int_t psymbfact_LUXpandMem
-(int_t, int_t, int_t, int_t, int_t, int_t, int_t, int_t, 
+(int, int_t, int_t, int_t, int_t, int, int, int, 
  Pslu_freeable_t *, Llu_symbfact_t *,  vtcsInfo_symbfact_t *, psymbfact_stat_t *);
 
 extern int_t psymbfact_LUXpand
@@ -1074,10 +1179,15 @@ int superlu_sort_perm (const void *arg1, const void *arg2)
 #endif
 
 #ifdef GPU_ACC   /* GPU related */
-extern void gemm_division_cpu_gpu (int *, int *, int *, int,
-				   int, int, int *, int);
-extern int_t get_cublas_nb ();
-extern int_t get_num_cuda_streams ();
+extern void gemm_division_cpu_gpu (superlu_dist_options_t *,
+				   int *, int *, int *, int,
+				   int, int, int *, int, int_t);
+extern int_t get_gpublas_nb ();
+extern int_t get_num_gpu_streams ();
+extern int getnGPUStreams();
+extern int get_mpi_process_per_gpu ();
+/*to print out various statistics from GPU activities*/
+extern void printGPUStats(int nsupers, SuperLUStat_t *stat, gridinfo3d_t*);
 #endif
 
 extern double estimate_cpu_time(int m, int n , int k);
@@ -1095,6 +1205,7 @@ extern int get_acc_offload();
 extern void  print_panel_seg_dist(int_t, int_t, int_t, int_t, int_t *, int_t *);
 extern void  check_repfnz_dist(int_t, int_t, int_t, int_t *);
 extern int_t CheckZeroDiagonal(int_t, int_t *, int_t *, int_t *);
+extern int   check_perm_dist(char *what, int_t n, int_t *perm);
 extern void  PrintDouble5(char *, int_t, double *);
 extern void  PrintInt10(char *, int_t, int_t *);
 extern void  PrintInt32(char *, int, int *);
@@ -1106,43 +1217,40 @@ extern int   file_PrintLong10(FILE *, char *, int_t, int_t *);
 
 #ifndef __SUPERLU_ASYNC_TREE /* allow multiple inclusions */
 #define __SUPERLU_ASYNC_TREE
-typedef void* BcTree;
-typedef void* RdTree;
-typedef void* StdList;
+typedef struct
+{
+    MPI_Request sendRequests_[2];
+    MPI_Comm comm_;
+    int myRoot_;
+    int destCnt_;
+    int myDests_[2];
+    int myRank_;
+    int msgSize_;
+    int tag_;
+    yes_no_t empty_;
+    MPI_Datatype type_;
+} C_Tree;
+
+#ifndef DEG_TREE
+#define DEG_TREE 2
 #endif
 
-// typedef enum {NO, YES}  yes_no_t;
-extern RdTree   RdTree_Create(MPI_Comm comm, int* ranks, int rank_cnt, int msgSize, double rseed, char precision);  
-extern void   	RdTree_Destroy(RdTree Tree, char precision);
-extern void 	RdTree_SetTag(RdTree Tree, int tag, char precision);
-extern yes_no_t RdTree_IsRoot(RdTree Tree, char precision);
-extern void 	RdTree_forwardMessageSimple(RdTree Tree, void* localBuffer, int msgSize, char precision);
-extern void 	RdTree_allocateRequest(RdTree Tree, char precision);
-extern int  	RdTree_GetDestCount(RdTree Tree, char precision);
-extern int  	RdTree_GetMsgSize(RdTree Tree, char precision);
-extern void 	RdTree_waitSendRequest(RdTree Tree, char precision);
+#endif
 
-extern BcTree   BcTree_Create(MPI_Comm comm, int* ranks, int rank_cnt, int msgSize, double rseed, char precision);  
-extern void   	BcTree_Destroy(BcTree Tree, char precision);
-extern void 	BcTree_SetTag(BcTree Tree, int tag, char precision);
-extern yes_no_t BcTree_IsRoot(BcTree Tree, char precision);
-extern void 	BcTree_forwardMessageSimple(BcTree Tree, void* localBuffer, int msgSize, char precision);
-extern void 	BcTree_allocateRequest(BcTree Tree, char precision);
-extern int 		BcTree_getDestCount(BcTree Tree, char precision); 
-extern int 		BcTree_GetMsgSize(BcTree Tree, char precision); 
-extern void 	BcTree_waitSendRequest(BcTree Tree, char precision);
- 
-extern StdList 	StdList_Init();
-extern void 	StdList_Pushback(StdList lst, int_t dat);
-extern void 	StdList_Pushfront(StdList lst, int_t dat);
-extern int_t 		StdList_Popfront(StdList lst);
-extern yes_no_t StdList_Find(StdList lst, int_t dat);
-extern int_t 	   	StdList_Size(StdList lst);
-yes_no_t 		StdList_Empty(StdList lst);
+extern void C_RdTree_Create(C_Tree* tree, MPI_Comm comm, int* ranks, int rank_cnt, int msgSize, char precision);
+extern void C_RdTree_Nullify(C_Tree* tree);
+extern yes_no_t C_RdTree_IsRoot(C_Tree* tree);
+extern void C_RdTree_forwardMessageSimple(C_Tree* Tree, void* localBuffer, int msgSize);
+extern void C_RdTree_waitSendRequest(C_Tree* Tree);
 
+extern void C_BcTree_Create(C_Tree* tree, MPI_Comm comm, int* ranks, int rank_cnt, int msgSize, char precision);
+extern void C_BcTree_Nullify(C_Tree* tree);
+extern yes_no_t C_BcTree_IsRoot(C_Tree* tree);
+extern void C_BcTree_forwardMessageSimple(C_Tree* tree, void* localBuffer, int msgSize);
+extern void C_BcTree_waitSendRequest(C_Tree* tree);
 
 /*==== For 3D code ====*/
-    
+
 extern void DistPrint(char* function_name,  double value, char* Units, gridinfo_t* grid);
 extern void DistPrint3D(char* function_name,  double value, char* Units, gridinfo3d_t* grid3d);
 extern void treeImbalance3D(gridinfo3d_t *grid3d, SCT_t* SCT);
@@ -1244,19 +1352,26 @@ extern sForest_t**  getGreedyLoadBalForests( int_t maxLvl, int_t nsupers, int_t*
 extern sForest_t**  getForests( int_t maxLvl, int_t nsupers, int_t*setree, treeList_t* treeList);
 
     /* from trfAux.h */
-extern int_t getBigUSize(int_t nsupers, gridinfo_t *grid, int_t **Lrowind_bc_ptr);
+extern int_t getBigUSize(superlu_dist_options_t *, int_t nsupers,
+			 gridinfo_t *grid, int_t **Lrowind_bc_ptr);
 extern void getSCUweight(int_t nsupers, treeList_t* treeList, int_t* xsup,
 			 int_t** Lrowind_bc_ptr, int_t** Ufstnz_br_ptr,
 			 gridinfo3d_t * grid3d);
+extern int Wait_LUDiagSend(int_t k, MPI_Request *U_diag_blk_send_req,
+			   MPI_Request *L_diag_blk_send_req,
+			   gridinfo_t *grid, SCT_t *SCT);
+
 extern int getNsupers(int n, Glu_persist_t *Glu_persist);
 extern int set_tag_ub();
 extern int getNumThreads(int);
 extern int_t num_full_cols_U(int_t kk, int_t **Ufstnz_br_ptr, int_t *xsup,
 			     gridinfo_t *, int_t *, int_t *);
+
 #if 0 // Sherry: conflicting with existing routine
 extern int_t estimate_bigu_size(int_t nsupers, int_t ldt, int_t**Ufstnz_br_ptr,
 				Glu_persist_t *, gridinfo_t*, int_t* perm_u);
 #endif
+
 extern int_t* getFactPerm(int_t);
 extern int_t* getFactIperm(int_t*, int_t);
 
@@ -1297,8 +1412,18 @@ extern int_t* getMyNodeCounts(int_t maxLvl, int_t* myTreeIdxs, int_t* gNodeCount
 extern int_t checkIntVector3d(int_t* vec, int_t len,  gridinfo3d_t* grid3d);
 extern int_t reduceStat(PhaseType PHASE, SuperLUStat_t *stat, gridinfo3d_t * grid3d);
 
-  extern int getnCudaStreams();
-  extern int get_mpi_process_per_gpu ();
+    /* from communication_aux.h */
+extern int_t Wait_LSend(int_t k, gridinfo_t *grid, int **ToSendR,
+			MPI_Request *s, SCT_t*);
+extern int_t Wait_USend(MPI_Request *, gridinfo_t *, SCT_t *);
+extern int_t Check_LRecv(MPI_Request*, int* msgcnt);
+extern int_t Wait_UDiagBlockSend(MPI_Request *, gridinfo_t *, SCT_t *);
+extern int_t Wait_LDiagBlockSend(MPI_Request *, gridinfo_t *, SCT_t *);
+extern int_t Wait_UDiagBlock_Recv(MPI_Request *, SCT_t *);
+extern int_t Test_UDiagBlock_Recv(MPI_Request *, SCT_t *);
+extern int_t Wait_LDiagBlock_Recv(MPI_Request *, SCT_t *);
+extern int_t Test_LDiagBlock_Recv(MPI_Request *, SCT_t *);
+extern int_t LDiagBlockRecvWait( int_t k, int_t* factored_U, MPI_Request *, gridinfo_t *);
 
 /*=====================*/
 
