@@ -1,5 +1,11 @@
 /* superlu_dist_config.h.in */
 
+/* Enable CUDA */
+/* #undef HAVE_CUDA */
+
+/* Enable HIP */
+/* #undef HAVE_HIP */
+
 /* Enable parmetis */
 #define HAVE_PARMETIS TRUE
 
@@ -7,12 +13,11 @@
 /* #undef SLU_HAVE_LAPACK */
 
 /* Enable CombBLAS */
-/* #undef HAVE_COMBBLAS */
+#define HAVE_COMBBLAS TRUE
 
 /* enable 64bit index mode */
 /* #undef XSDK_INDEX_SIZE */
 
 #if (XSDK_INDEX_SIZE == 64)
 #define _LONGINT 1
-
 #endif
