@@ -102,13 +102,14 @@ int zLluBufFreeArr(int_t numLA, zLUValSubBuf_t **LUvsbs)
 }
 
 
-int_t zinitScuBufs(int_t ldt, int_t num_threads, int_t nsupers,
+int_t zinitScuBufs(superlu_dist_options_t *options,
+                  int_t ldt, int_t num_threads, int_t nsupers,
                   zscuBufs_t* scuBufs,
                   zLUstruct_t* LUstruct,
                   gridinfo_t * grid)
 {
     scuBufs->bigV = zgetBigV(ldt, num_threads);
-    scuBufs->bigU = zgetBigU(nsupers, grid, LUstruct);
+    scuBufs->bigU = zgetBigU(options, nsupers, grid, LUstruct);
     return 0;
 }
 
